@@ -9,12 +9,11 @@
 #include <time.h>
 #include <wlr/config.h>
 #include <wlr/types/wlr_box.h>
-#include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_export_dmabuf_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
-#include <wlr/types/wlr_gtk_primary_selection.h>
+#include <wlr/types/wlr_primary_selection_v1.h>
 #include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
@@ -23,12 +22,16 @@
 #include <wlr/types/wlr_pointer_constraints_v1.h>
 #include <wlr/types/wlr_server_decoration.h>
 #include <wlr/types/wlr_tablet_v2.h>
+#include <wlr/types/wlr_viewporter.h>
 #include <wlr/types/wlr_xcursor_manager.h>
+#include <wlr/types/wlr_xdg_foreign_registry.h>
+#include <wlr/types/wlr_xdg_foreign_v1.h>
+#include <wlr/types/wlr_xdg_foreign_v2.h>
 #include <wlr/types/wlr_xdg_output_v1.h>
 #include <wlr/types/wlr_xdg_output_v1.h>
 #include <wlr/types/wlr_xdg_shell.h>
-#include <wlr/util/log.h>
 #include <wlr/version.h>
+#include "cursor.h"
 #include "layers.h"
 #include "output.h"
 #include "seat.h"
@@ -37,8 +40,17 @@
 #include "view.h"
 #include "virtual.h"
 #include "xcursor.h"
+#include "xdg-activation-v1.h"
 #include "wlr-layer-shell-unstable-v1-protocol.h"
 
+#include "xdg-surface.h"
+
+/**
+ * PhocDesktop:
+ *
+ * Desktop singleton
+ */
+
 enum {
   PROP_0,
   PROP_CONFIG,
@@ -88,9 +100,9 @@ phoc_desktop_get_property (GObject    *object,
 }
 
 
-static bool view_at(struct roots_view *view, double lx, double ly,
+static bool view_at(PhocView *view, double lx, double ly,
 		struct wlr_surface **surface, double *sx, double *sy) {
-	if (view->wlr_surface == NULL) {
+	if (!phoc_view_is_mapped (view)) {
 		return false;
 	}
 
@@ -100,14 +112,13 @@ static bool view_at(struct roots_view *view, double lx, double ly,
 	double _sx, _sy;
 	struct wlr_surface *_surface = NULL;
 	switch (view->type) {
-	case ROOTS_XDG_SHELL_VIEW:;
-		struct roots_xdg_surface *xdg_surface =
-			roots_xdg_surface_from_view(view);
+	case PHOC_XDG_SHELL_VIEW:;
+		PhocXdgSurface *xdg_surface = phoc_xdg_surface_from_view(view);
 		_surface = wlr_xdg_surface_surface_at(xdg_surface->xdg_surface,
 			view_sx, view_sy, &_sx, &_sy);
 		break;
 #ifdef PHOC_XWAYLAND
-	case ROOTS_XWAYLAND_VIEW:
+	case PHOC_XWAYLAND_VIEW:
 		_surface = wlr_surface_surface_at(view->wlr_surface,
 			view_sx, view_sy, &_sx, &_sy);
 		break;
@@ -132,10 +143,10 @@ static bool view_at(struct roots_view *view, double lx, double ly,
 	return false;
 }
 
-static struct roots_view *desktop_view_at(PhocDesktop *desktop,
+static PhocView *desktop_view_at(PhocDesktop *desktop,
 		double lx, double ly, struct wlr_surface **surface,
 		double *sx, double *sy) {
-	struct roots_view *view;
+	PhocView *view;
 	wl_list_for_each(view, &desktop->views, link) {
 		if (phoc_desktop_view_is_visible(desktop, view) && view_at(view, lx, ly, surface, sx, sy)) {
 			return view;
@@ -147,34 +158,34 @@ static struct roots_view *desktop_view_at(PhocDesktop *desktop,
 static struct wlr_surface *layer_surface_at(struct wl_list *layer, double ox,
                                              double oy, double *sx, double *sy)
 {
-	struct roots_layer_surface *roots_surface;
+	PhocLayerSurface *layer_surface;
 
-	wl_list_for_each_reverse(roots_surface, layer, link) {
-		if (roots_surface->layer_surface->current.exclusive_zone <= 0) {
+	wl_list_for_each_reverse(layer_surface, layer, link) {
+		if (layer_surface->layer_surface->current.exclusive_zone <= 0) {
 			continue;
 		}
 
-		double _sx = ox - roots_surface->geo.x;
-		double _sy = oy - roots_surface->geo.y;
+		double _sx = ox - layer_surface->geo.x;
+		double _sy = oy - layer_surface->geo.y;
 
 		struct wlr_surface *sub = wlr_layer_surface_v1_surface_at(
-			roots_surface->layer_surface, _sx, _sy, sx, sy);
+			layer_surface->layer_surface, _sx, _sy, sx, sy);
 
 		if (sub) {
 			return sub;
 		}
 	}
 
-	wl_list_for_each(roots_surface, layer, link) {
-		if (roots_surface->layer_surface->current.exclusive_zone > 0) {
+	wl_list_for_each(layer_surface, layer, link) {
+		if (layer_surface->layer_surface->current.exclusive_zone > 0) {
 			continue;
 		}
 
-		double _sx = ox - roots_surface->geo.x;
-		double _sy = oy - roots_surface->geo.y;
+		double _sx = ox - layer_surface->geo.x;
+		double _sy = oy - layer_surface->geo.y;
 
 		struct wlr_surface *sub = wlr_layer_surface_v1_surface_at(
-			roots_surface->layer_surface, _sx, _sy, sx, sy);
+			layer_surface->layer_surface, _sx, _sy, sx, sy);
 
 		if (sub) {
 			return sub;
@@ -184,9 +195,24 @@ static struct wlr_surface *layer_surface_at(struct wl_list *layer, double ox,
 	return NULL;
 }
 
+/**
+ * phoc_desktop_surface_at:
+ * @desktop: The `PhocDesktop` to look the surface up for
+ * @lx: X coordinate the surface to look up at in layout coordinates
+ * @ly: Y coordinate the surface to look up at in layout coordinates
+ * @sx: (out) (not nullable): Surface-local x coordinate
+ * @sy: (out) (not nullable): Surface-local y coordinate
+ * @view: (out) (optional): The corresponding [class@Phoc.View]
+ *
+ * Looks up the surface at `lx,ly` and returns the topmost surface at
+ * that position (if any) and the surface-local coordinates of `sx,sy`
+ * on that surface.
+ *
+ * Returns: (nullable): The `struct wlr_surface`
+ */
 struct wlr_surface *phoc_desktop_surface_at(PhocDesktop *desktop,
 		double lx, double ly, double *sx, double *sy,
-		struct roots_view **view) {
+		PhocView **view) {
 	struct wlr_surface *surface = NULL;
 	struct wlr_output *wlr_output =
 		wlr_output_layout_output_at(desktop->layout, lx, ly);
@@ -231,7 +257,7 @@ struct wlr_surface *phoc_desktop_surface_at(PhocDesktop *desktop,
 		}
 	}
 
-	struct roots_view *_view;
+	PhocView *_view;
 	if ((_view = desktop_view_at(desktop, lx, ly, &surface, sx, sy))) {
 		if (view) {
 			*view = _view;
@@ -253,9 +279,9 @@ struct wlr_surface *phoc_desktop_surface_at(PhocDesktop *desktop,
 }
 
 gboolean
-phoc_desktop_view_is_visible (PhocDesktop *desktop, struct roots_view *view)
+phoc_desktop_view_is_visible (PhocDesktop *desktop, PhocView *view)
 {
-  if (!view->wlr_surface) {
+  if (!phoc_view_is_mapped (view)) {
     return false;
   }
 
@@ -270,18 +296,18 @@ phoc_desktop_view_is_visible (PhocDesktop *desktop, struct roots_view *view)
     return true;
   }
 
-  struct roots_view *top_view = wl_container_of (desktop->views.next, view, link);
+  PhocView *top_view = wl_container_of (desktop->views.next, view, link);
 
 #ifdef PHOC_XWAYLAND
-  // XWayland parent relations can be complicated and aren't described by roots_view
+  // XWayland parent relations can be complicated and aren't described by PhocView
   // relationships very well at the moment, so just make all XWayland windows visible
   // when some XWayland window is active for now
-  if (view->type == ROOTS_XWAYLAND_VIEW && top_view->type == ROOTS_XWAYLAND_VIEW) {
+  if (view->type == PHOC_XWAYLAND_VIEW && top_view->type == PHOC_XWAYLAND_VIEW) {
     return true;
   }
 #endif
 
-  struct roots_view *v = top_view;
+  PhocView *v = top_view;
   while (v) {
     if (v == view) {
       return true;
@@ -302,7 +328,7 @@ handle_layout_change (struct wl_listener *listener, void *data)
   struct wlr_output *center_output;
   struct wlr_box *center_output_box;
   double center_x, center_y;
-  struct roots_view *view;
+  PhocView *view;
   PhocOutput *output;
 
   self = wl_container_of (listener, self, layout_change);
@@ -332,11 +358,13 @@ handle_layout_change (struct wl_listener *listener, void *data)
 static void input_inhibit_activate(struct wl_listener *listener, void *data) {
 	PhocDesktop *desktop = wl_container_of(
 			listener, desktop, input_inhibit_activate);
-	struct roots_seat *seat;
 	PhocServer *server = phoc_server_get_default ();
 
-	wl_list_for_each(seat, &server->input->seats, link) {
-		roots_seat_set_exclusive_client(seat,
+	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+		PhocSeat *seat = PHOC_SEAT (elem->data);
+
+		g_assert (PHOC_IS_SEAT (seat));
+		phoc_seat_set_exclusive_client(seat,
 				desktop->input_inhibit->active_client);
 	}
 }
@@ -344,21 +372,23 @@ static void input_inhibit_activate(struct wl_listener *listener, void *data) {
 static void input_inhibit_deactivate(struct wl_listener *listener, void *data) {
 	PhocDesktop *desktop = wl_container_of(
 			listener, desktop, input_inhibit_deactivate);
-	struct roots_seat *seat;
 	PhocServer *server = phoc_server_get_default ();
 
-	wl_list_for_each(seat, &server->input->seats, link) {
-		roots_seat_set_exclusive_client(seat, NULL);
+	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+		PhocSeat *seat = PHOC_SEAT (elem->data);
+
+		g_assert (PHOC_IS_SEAT (seat));
+		phoc_seat_set_exclusive_client(seat, NULL);
 	}
 }
 
 static void handle_constraint_destroy(struct wl_listener *listener,
 		void *data) {
-	struct roots_pointer_constraint *constraint =
+	PhocPointerConstraint *constraint =
 		wl_container_of(listener, constraint, destroy);
 	struct wlr_pointer_constraint_v1 *wlr_constraint = data;
-	struct roots_seat *seat = wlr_constraint->seat->data;
-	struct roots_cursor *cursor = roots_seat_get_cursor (seat);
+	PhocSeat *seat = wlr_constraint->seat->data;
+	PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
 	wl_list_remove(&constraint->destroy.link);
 
@@ -370,7 +400,7 @@ static void handle_constraint_destroy(struct wl_listener *listener,
 		if (wlr_constraint->current.committed &
 				WLR_POINTER_CONSTRAINT_V1_STATE_CURSOR_HINT &&
 				cursor->pointer_view) {
-			struct roots_view *view = cursor->pointer_view->view;
+			PhocView *view = cursor->pointer_view->view;
 			double lx = view->box.x + wlr_constraint->current.cursor_hint.x;
 			double ly = view->box.y + wlr_constraint->current.cursor_hint.y;
 
@@ -385,11 +415,11 @@ static void handle_pointer_constraint(struct wl_listener *listener,
 		void *data) {
 	PhocServer *server = phoc_server_get_default ();
 	struct wlr_pointer_constraint_v1 *wlr_constraint = data;
-	struct roots_seat *seat = wlr_constraint->seat->data;
-	struct roots_cursor *cursor = roots_seat_get_cursor (seat);
+	PhocSeat *seat = wlr_constraint->seat->data;
+	PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
-	struct roots_pointer_constraint *constraint =
-		calloc(1, sizeof(struct roots_pointer_constraint));
+	PhocPointerConstraint *constraint =
+		calloc(1, sizeof(PhocPointerConstraint));
 	constraint->constraint = wlr_constraint;
 
 	constraint->destroy.notify = handle_constraint_destroy;
@@ -402,7 +432,7 @@ static void handle_pointer_constraint(struct wl_listener *listener,
 
 	if (surface == wlr_constraint->surface) {
 		assert(!cursor->active_constraint);
-		roots_cursor_constrain(cursor, wlr_constraint, sx, sy);
+		phoc_cursor_constrain(cursor, wlr_constraint, sx, sy);
 	}
 }
 
@@ -473,7 +503,23 @@ void handle_xwayland_ready(struct wl_listener *listener, void *data) {
 
   xcb_disconnect (xcb_conn);
 }
-#endif
+
+#ifdef PHOC_HAVE_WLR_REMOVE_STARTUP_INFO
+static
+void handle_xwayland_remove_startup_id(struct wl_listener *listener, void *data) {
+  PhocDesktop *desktop = wl_container_of (
+        listener, desktop, xwayland_remove_startup_id);
+  struct wlr_xwayland_remove_startup_info_event *ev = data;
+
+  g_assert (PHOC_IS_DESKTOP (desktop));
+  g_assert (ev->id);
+
+  phoc_phosh_private_notify_startup_id (desktop->phosh,
+                                        ev->id,
+                                        PHOSH_PRIVATE_STARTUP_TRACKER_PROTOCOL_X11);
+}
+#endif /* PHOC_HAVE_WLR_REMOVE_STARTUP_INFO */
+#endif /* PHOC_XWAYLAND */
 
 static void
 handle_output_destroy (PhocOutput *destroyed_output)
@@ -507,12 +553,64 @@ handle_new_output (struct wl_listener *listener, void *data)
 			  NULL);
 }
 
+
+static void
+phoc_desktop_setup_xwayland (PhocDesktop *self)
+{
+#ifdef PHOC_XWAYLAND
+  const char *cursor_default = PHOC_XCURSOR_DEFAULT;
+  PhocConfig *config = self->config;
+  PhocServer *server = phoc_server_get_default ();
+
+  self->xcursor_manager = wlr_xcursor_manager_create(NULL, PHOC_XCURSOR_SIZE);
+  g_return_if_fail (self->xcursor_manager);
+
+  if (config->xwayland) {
+    self->xwayland = wlr_xwayland_create(server->wl_display,
+					 server->compositor, config->xwayland_lazy);
+    if (!self->xwayland) {
+      g_critical ("Failed to initialize Xwayland");
+      g_unsetenv ("DISPLAY");
+      return;
+    }
+
+    wl_signal_add(&self->xwayland->events.new_surface,
+		  &self->xwayland_surface);
+    self->xwayland_surface.notify = handle_xwayland_surface;
+
+    wl_signal_add(&self->xwayland->events.ready,
+		  &self->xwayland_ready);
+    self->xwayland_ready.notify = handle_xwayland_ready;
+
+#ifdef PHOC_HAVE_WLR_REMOVE_STARTUP_INFO
+    wl_signal_add(&self->xwayland->events.remove_startup_info,
+		  &self->xwayland_remove_startup_id);
+    self->xwayland_remove_startup_id.notify = handle_xwayland_remove_startup_id;
+#endif
+
+    g_setenv ("DISPLAY", self->xwayland->display_name, true);
+
+    if (!wlr_xcursor_manager_load(self->xcursor_manager, 1))
+      g_critical ("Cannot load XWayland XCursor theme");
+
+    struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
+								  self->xcursor_manager, cursor_default, 1);
+    if (xcursor != NULL) {
+      struct wlr_xcursor_image *image = xcursor->images[0];
+      wlr_xwayland_set_cursor(self->xwayland, image->buffer,
+			      image->width * 4, image->width, image->height, image->hotspot_x,
+			      image->hotspot_y);
+    }
+  }
+#endif
+}
+
+
 static void
 phoc_desktop_constructed (GObject *object)
 {
   PhocDesktop *self = PHOC_DESKTOP (object);
   PhocServer *server = phoc_server_get_default ();
-  struct roots_config *config = self->config;
 
   G_OBJECT_CLASS (phoc_desktop_parent_class)->constructed (object);
 
@@ -527,9 +625,6 @@ phoc_desktop_constructed (GObject *object)
   self->layout_change.notify = handle_layout_change;
   wl_signal_add(&self->layout->events.change, &self->layout_change);
 
-  self->compositor = wlr_compositor_create(server->wl_display,
-					   server->renderer);
-
   self->xdg_shell = wlr_xdg_shell_create(server->wl_display);
   wl_signal_add(&self->xdg_shell->events.new_surface,
 		&self->xdg_shell_surface);
@@ -542,54 +637,12 @@ phoc_desktop_constructed (GObject *object)
 
   self->tablet_v2 = wlr_tablet_v2_create(server->wl_display);
 
-  const char *cursor_theme = NULL;
-#ifdef PHOC_XWAYLAND
-  const char *cursor_default = ROOTS_XCURSOR_DEFAULT;
-#endif
-
   char cursor_size_fmt[16];
   snprintf(cursor_size_fmt, sizeof(cursor_size_fmt),
-	   "%d", ROOTS_XCURSOR_SIZE);
-  setenv("XCURSOR_SIZE", cursor_size_fmt, 1);
-  if (cursor_theme != NULL) {
-    setenv("XCURSOR_THEME", cursor_theme, 1);
-  }
+	   "%d", PHOC_XCURSOR_SIZE);
+  g_setenv("XCURSOR_SIZE", cursor_size_fmt, 1);
 
-#ifdef PHOC_XWAYLAND
-  self->xcursor_manager = wlr_xcursor_manager_create(cursor_theme,
-						     ROOTS_XCURSOR_SIZE);
-  g_return_if_fail (self->xcursor_manager);
-
-  if (config->xwayland) {
-    self->xwayland = wlr_xwayland_create(server->wl_display,
-					 self->compositor, config->xwayland_lazy);
-    wl_signal_add(&self->xwayland->events.new_surface,
-		  &self->xwayland_surface);
-    self->xwayland_surface.notify = handle_xwayland_surface;
-
-    wl_signal_add(&self->xwayland->events.ready,
-		  &self->xwayland_ready);
-    self->xwayland_ready.notify = handle_xwayland_ready;
-
-    setenv("DISPLAY", self->xwayland->display_name, true);
-
-#if (WLR_VERSION_MAJOR > 0 || WLR_VERSION_MINOR > 10)
-    if (!wlr_xcursor_manager_load(self->xcursor_manager, 1)) {
-#else
-    if (wlr_xcursor_manager_load(self->xcursor_manager, 1)) {
-#endif
-      wlr_log(WLR_ERROR, "Cannot load XWayland XCursor theme");
-    }
-    struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
-								  self->xcursor_manager, cursor_default, 1);
-    if (xcursor != NULL) {
-      struct wlr_xcursor_image *image = xcursor->images[0];
-      wlr_xwayland_set_cursor(self->xwayland, image->buffer,
-			      image->width * 4, image->width, image->height, image->hotspot_x,
-			      image->hotspot_y);
-    }
-  }
-#endif
+  phoc_desktop_setup_xwayland (self);
 
   self->gamma_control_manager_v1 = wlr_gamma_control_manager_v1_create(server->wl_display);
   self->export_dmabuf_manager_v1 =
@@ -600,7 +653,7 @@ phoc_desktop_constructed (GObject *object)
 						 WLR_SERVER_DECORATION_MANAGER_MODE_CLIENT);
   self->idle = wlr_idle_create(server->wl_display);
   self->primary_selection_device_manager =
-    wlr_gtk_primary_selection_device_manager_create(server->wl_display);
+    wlr_primary_selection_v1_device_manager_create(server->wl_display);
   self->input_inhibit =
     wlr_input_inhibit_manager_create(server->wl_display);
   self->input_inhibit_activate.notify = input_inhibit_activate;
@@ -616,7 +669,13 @@ phoc_desktop_constructed (GObject *object)
   self->text_input = wlr_text_input_manager_v3_create(server->wl_display);
 
   self->gtk_shell = phoc_gtk_shell_create(self, server->wl_display);
-  self->phosh = phosh_create(self, server->wl_display);
+  self->phosh = phoc_phosh_private_new ();
+
+  self->xdg_activation_v1 = wlr_xdg_activation_v1_create (server->wl_display);
+  self->xdg_activation_v1_request_activate.notify = phoc_xdg_activation_v1_handle_request_activate;
+  wl_signal_add (&self->xdg_activation_v1->events.request_activate,
+                 &self->xdg_activation_v1_request_activate);
+
   self->virtual_keyboard = wlr_virtual_keyboard_manager_v1_create(
 								  server->wl_display);
   wl_signal_add(&self->virtual_keyboard->events.new_virtual_keyboard,
@@ -635,6 +694,12 @@ phoc_desktop_constructed (GObject *object)
   wl_signal_add(&self->xdg_decoration_manager->events.new_toplevel_decoration,
 		&self->xdg_toplevel_decoration);
   self->xdg_toplevel_decoration.notify = handle_xdg_toplevel_decoration;
+  wlr_viewporter_create(server->wl_display);
+
+  struct wlr_xdg_foreign_registry *foreign_registry =
+                wlr_xdg_foreign_registry_create(server->wl_display);
+  wlr_xdg_foreign_v1_create(server->wl_display, foreign_registry);
+  wlr_xdg_foreign_v2_create(server->wl_display, foreign_registry);
 
   self->pointer_constraints =
     wlr_pointer_constraints_v1_create(server->wl_display);
@@ -684,14 +749,41 @@ phoc_desktop_finalize (GObject *object)
 {
   PhocDesktop *self = PHOC_DESKTOP (object);
 
+  /* TODO: currently destroys the backend before the desktop */
+  //wl_list_remove (&self->new_output.link);
+  wl_list_remove (&self->layout_change.link);
+  wl_list_remove (&self->xdg_shell_surface.link);
+  wl_list_remove (&self->layer_shell_surface.link);
+  wl_list_remove (&self->xdg_toplevel_decoration.link);
+  wl_list_remove (&self->input_inhibit_activate.link);
+  wl_list_remove (&self->input_inhibit_deactivate.link);
+  wl_list_remove (&self->virtual_keyboard_new.link);
+  wl_list_remove (&self->virtual_pointer_new.link);
+  wl_list_remove (&self->pointer_constraint.link);
+  wl_list_remove (&self->output_manager_apply.link);
+  wl_list_remove (&self->output_manager_test.link);
+  wl_list_remove (&self->output_power_manager_set_mode.link);
+  wl_list_remove (&self->xdg_activation_v1_request_activate.link);
+
 #ifdef PHOC_XWAYLAND
+  /* Disconnect XWayland listener before shutting it down */
+  if (self->xwayland) {
+    wl_list_remove (&self->xwayland_surface.link);
+    wl_list_remove (&self->xwayland_ready.link);
+#ifdef PHOC_HAVE_WLR_REMOVE_STARTUP_INFO
+    wl_list_remove (&self->xwayland_remove_startup_id.link);
+#endif
+  }
+
+  g_clear_pointer (&self->xcursor_manager, wlr_xcursor_manager_destroy);
   // We need to shutdown Xwayland before disconnecting all clients, otherwise
   // wlroots will restart it automatically.
   g_clear_pointer (&self->xwayland, wlr_xwayland_destroy);
 #endif
 
-  g_clear_pointer (&self->phosh, phosh_destroy);
+  g_clear_object (&self->phosh);
   g_clear_pointer (&self->gtk_shell, phoc_gtk_shell_destroy);
+  g_clear_pointer (&self->layout, wlr_output_layout_destroy);
 
   g_hash_table_remove_all (self->input_output_map);
   g_hash_table_unref (self->input_output_map);
@@ -733,7 +825,7 @@ phoc_desktop_init (PhocDesktop *self)
 
 
 PhocDesktop *
-phoc_desktop_new (struct roots_config *config)
+phoc_desktop_new (PhocConfig *config)
 {
   return g_object_new (PHOC_TYPE_DESKTOP, "config", config, NULL);
 }
@@ -742,16 +834,26 @@ phoc_desktop_new (struct roots_config *config)
 /**
  * phoc_desktop_toggle_output_blank:
  *
- * Blank or unblank all outputs depending on the current state
+ * Blank or unblank all outputs depending on the current state.
+ * Brings all outputs into consistent state (either all blanked
+ * or all enabled).
  */
 void
 phoc_desktop_toggle_output_blank (PhocDesktop *self)
 {
   PhocOutput *output;
 
+  gboolean enable = true;
   wl_list_for_each(output, &self->outputs, link) {
-    gboolean enable = !output->wlr_output->enabled;
+    if (output->wlr_output->enabled) {
+      enable = false;
+      break;
+    }
+  }
 
+  wl_list_for_each(output, &self->outputs, link) {
+    if (!wlr_output_layout_get (self->layout, output->wlr_output))
+      continue;
     wlr_output_enable (output->wlr_output, enable);
     wlr_output_commit (output->wlr_output);
     if (enable)
@@ -767,7 +869,14 @@ phoc_desktop_toggle_output_blank (PhocDesktop *self)
 void
 phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
 {
-  struct roots_view *view;
+  PhocView *view;
+  PhocServer *server = phoc_server_get_default();
+
+  if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_AUTO_MAXIMIZE)) {
+    if (enable == FALSE)
+      g_info ("Not disabling auto-maximize due to `auto-maximize` debug flag");
+    enable = TRUE;
+  }
 
   g_debug ("auto-maximize: %d", enable);
   self->maximize = enable;
@@ -808,3 +917,59 @@ phoc_desktop_get_scale_to_fit (PhocDesktop *self)
 {
     return self->scale_to_fit;
 }
+
+
+/**
+ * phoc_desktop_find_output:
+ * @self: The desktop
+ * @make: The output's make / vendor
+ * @model: The output's model / product
+ * @serial: The output's serial number
+ *
+ * Find an output by make, model and serial
+ *
+ * Returns: (transfer none) (nullable): The matching output or
+ *  %NULL if no output matches.
+ */
+PhocOutput *
+phoc_desktop_find_output (PhocDesktop *self,
+                          const char  *make,
+                          const char  *model,
+                          const char  *serial)
+{
+  PhocOutput *output;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  wl_list_for_each (output, &self->outputs, link) {
+    if (phoc_output_is_match (output, make, model, serial))
+        return output;
+  }
+
+  return NULL;
+}
+
+
+/**
+ * phoc_desktop_get_builtin_output:
+ *
+ * Get the built-in output. This assumes there's only one
+ * and returns the first.
+ *
+ * Returns: (transfer none) (nullable): The built-in output.
+ *  %NULL if there's no built-in output.
+ */
+PhocOutput *
+phoc_desktop_get_builtin_output (PhocDesktop *self)
+{
+  PhocOutput *output;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  wl_list_for_each (output, &self->outputs, link) {
+    if (phoc_output_is_builtin (output))
+        return output;
+  }
+
+  return NULL;
+}
