@@ -15,10 +15,11 @@
 #include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_surface.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
-#include <wlr/util/log.h>
+#include "cursor.h"
 #include "desktop.h"
 #include "layers.h"
 #include "output.h"
+#include "seat.h"
 #include "server.h"
 
 #define LAYER_SHELL_LAYER_COUNT 4
@@ -85,12 +86,15 @@ static void apply_exclusive(struct wlr_box *usable_area,
 	}
 }
 
-static void update_cursors(struct roots_layer_surface *roots_surface,
-		struct wl_list *seats /* struct roots_seat */) {
+static void update_cursors(PhocLayerSurface *roots_surface,
+		GSList *seats /* PhocSeat */) {
 	PhocServer *server = phoc_server_get_default ();
-	struct roots_seat *seat;
-	wl_list_for_each(seat, seats, link) {
-		struct roots_cursor *cursor = roots_seat_get_cursor(seat);
+
+	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+		PhocSeat *seat = PHOC_SEAT (elem->data);
+
+		g_assert (PHOC_IS_SEAT (seat));
+		PhocCursor *cursor = phoc_seat_get_cursor(seat);
 		double sx, sy;
 
 		struct wlr_surface *surface = phoc_desktop_surface_at(
@@ -100,21 +104,21 @@ static void update_cursors(struct roots_layer_surface *roots_surface,
 		if (surface == roots_surface->layer_surface->surface) {
 			struct timespec time;
 			if (clock_gettime(CLOCK_MONOTONIC, &time) == 0) {
-				roots_cursor_update_position(cursor,
-					time.tv_sec * 1000 + time.tv_nsec / 1000000);
+				phoc_cursor_update_position(cursor,
+							    time.tv_sec * 1000 + time.tv_nsec / 1000000);
 			} else {
-				wlr_log(WLR_ERROR, "Failed to get time, not updating"
-					"position. Errno: %s\n", strerror(errno));
+				g_critical ("Failed to get time, not updating position. Errno: %s\n",
+					    strerror(errno));
 			}
 		}
 	}
 }
 
 static void arrange_layer(struct wlr_output *output,
-		struct wl_list *seats /* struct *roots_seat */,
-		struct wl_list *list /* struct *roots_layer_surface */,
+		GSList *seats /* PhocSeat */,
+		struct wl_list *list /* PhocLayerSurface */,
 		struct wlr_box *usable_area, bool exclusive) {
-	struct roots_layer_surface *roots_surface;
+	PhocLayerSurface *roots_surface;
 	struct wlr_box full_area = { 0 };
 	wlr_output_effective_resolution(output,
 			&full_area.width, &full_area.height);
@@ -208,7 +212,7 @@ static void arrange_layer(struct wlr_output *output,
 
 struct osk_origin {
 	struct wlr_layer_surface_v1_state state;
-	struct roots_layer_surface *surface;
+	PhocLayerSurface *surface;
 	enum zwlr_layer_shell_v1_layer layer;
 };
 
@@ -216,7 +220,7 @@ static struct osk_origin find_osk(struct wl_list layers[LAYER_SHELL_LAYER_COUNT]
 	struct osk_origin origin = {0};
 	for (unsigned i = 0; i < LAYER_SHELL_LAYER_COUNT; i++) {
 		struct wl_list *list = &layers[i];
-		struct roots_layer_surface *roots_surface;
+		PhocLayerSurface *roots_surface;
 		wl_list_for_each(roots_surface, list, link) {
 			if (strcmp(roots_surface->layer_surface->namespace, "osk") == 0) {
 				origin.state = roots_surface->layer_surface->current;
@@ -242,98 +246,97 @@ static void change_osk(const struct osk_origin *osk, struct wl_list layers[LAYER
 	}
 }
 
-void arrange_layers(PhocOutput *output) {
-	struct wlr_box usable_area = { 0 };
-	PhocServer *server = phoc_server_get_default ();
+void
+phoc_layer_shell_arrange (PhocOutput *output)
+{
+  struct wlr_box usable_area = { 0 };
+  PhocServer *server = phoc_server_get_default ();
+  GSList *seats = phoc_input_get_seats (server->input);
+  enum zwlr_layer_shell_v1_layer layers[] = {
+    ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
+    ZWLR_LAYER_SHELL_V1_LAYER_TOP,
+    ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM,
+    ZWLR_LAYER_SHELL_V1_LAYER_BACKGROUND
+  };
 
-	wlr_output_effective_resolution(output->wlr_output,
-			&usable_area.width, &usable_area.height);
+  wlr_output_effective_resolution (output->wlr_output, &usable_area.width, &usable_area.height);
 
-	struct osk_origin osk_place = find_osk(output->layers);
-	if (osk_place.surface) {
-		bool osk_force_overlay = false;
-		struct roots_seat *seat;
-		wl_list_for_each(seat, &server->input->seats, link) {
-			if (seat->focused_layer && seat->focused_layer->client_pending.layer >=
-					osk_place.surface->layer_surface->client_pending.layer) {
-				osk_force_overlay = true;
-				break;
-			}
-		}
-		change_osk(&osk_place, output->layers, osk_force_overlay);
-	}
+  struct osk_origin osk_place = find_osk (output->layers);
+  if (osk_place.surface) {
+    bool osk_force_overlay = false;
 
-	// Arrange exclusive surfaces from top->bottom
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY],
-			&usable_area, true);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_TOP],
-			&usable_area, true);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM],
-			&usable_area, true);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_BACKGROUND],
-			&usable_area, true);
-	output->usable_area = usable_area;
+    for (GSList *elem = seats; elem; elem = elem->next) {
+      PhocSeat *seat = PHOC_SEAT (elem->data);
 
-	struct roots_view *view;
-	wl_list_for_each(view, &output->desktop->views, link) {
-		if (view_is_maximized(view)) {
-			view_arrange_maximized(view, NULL);
-		} else if (view_is_tiled(view)) {
-			view_arrange_tiled(view, NULL);
-		} else if (output->desktop->maximize) {
-			view_center(view);
-		}
-	}
+      g_assert (PHOC_IS_SEAT (seat));
+      if (seat->focused_layer && seat->focused_layer->client_pending.layer >= osk_place.surface->layer_surface->client_pending.layer) {
+        osk_force_overlay = true;
+        break;
+      }
+    }
+    change_osk (&osk_place, output->layers, osk_force_overlay);
+  }
 
-	// Arrange non-exlusive surfaces from top->bottom
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY],
-			&usable_area, false);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_TOP],
-			&usable_area, false);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM],
-			&usable_area, false);
-	arrange_layer(output->wlr_output, &server->input->seats,
-			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_BACKGROUND],
-			&usable_area, false);
+  // Arrange exclusive surfaces from top->bottom
+  for (size_t i = 0; i < G_N_ELEMENTS(layers); ++i)
+    arrange_layer (output->wlr_output, seats, &output->layers[layers[i]], &usable_area, true);
+  output->usable_area = usable_area;
 
-	// Find topmost keyboard interactive layer, if such a layer exists
-	uint32_t layers_above_shell[] = {
-		ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
-		ZWLR_LAYER_SHELL_V1_LAYER_TOP,
-	};
-	size_t nlayers = sizeof(layers_above_shell) / sizeof(layers_above_shell[0]);
-	struct roots_layer_surface *layer, *topmost = NULL;
-	for (size_t i = 0; i < nlayers; ++i) {
-		wl_list_for_each(layer,
-				&output->layers[layers_above_shell[i]], link) {
-			if (layer->layer_surface->current.keyboard_interactive
-					&& layer->layer_surface->mapped) {
-				topmost = layer;
-				break;
-			}
-		}
-		if (topmost != NULL) {
-			break;
-		}
-	}
+  PhocView *view;
+  wl_list_for_each (view, &output->desktop->views, link) {
+    if (view_is_maximized (view)) {
+      view_arrange_maximized (view, NULL);
+    } else if (view_is_tiled (view)) {
+      view_arrange_tiled (view, NULL);
+    } else if (output->desktop->maximize) {
+      view_center (view, NULL);
+    }
+  }
 
-	PhocInput *input = server->input;
-	struct roots_seat *seat;
-	wl_list_for_each(seat, &input->seats, link) {
-		roots_seat_set_focus_layer(seat,
-				topmost ? topmost->layer_surface : NULL);
-	}
+  // Arrange non-exlusive surfaces from top->bottom
+  for (size_t i = 0; i < G_N_ELEMENTS(layers); ++i)
+    arrange_layer (output->wlr_output, seats, &output->layers[layers[i]], &usable_area, false);
+}
+
+void
+phoc_layer_shell_update_focus (void)
+{
+  PhocServer *server = phoc_server_get_default ();
+  enum zwlr_layer_shell_v1_layer layers_above_shell[] = {
+    ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
+    ZWLR_LAYER_SHELL_V1_LAYER_TOP,
+  };
+  PhocLayerSurface *layer, *topmost = NULL;
+  // Find topmost keyboard interactive layer, if such a layer exists
+  // TODO: Make layer surface focus per-output based on cursor position
+  PhocOutput *output;
+  wl_list_for_each (output, &server->desktop->outputs, link) {
+    for (size_t i = 0; i < G_N_ELEMENTS(layers_above_shell); ++i) {
+      if (output->fullscreen_view && !output->force_shell_reveal) {
+        if (layers_above_shell[i] == ZWLR_LAYER_SHELL_V1_LAYER_TOP)
+          continue;
+      }
+      wl_list_for_each(layer, &output->layers[layers_above_shell[i]], link) {
+        if (layer->layer_surface->current.keyboard_interactive && layer->layer_surface->mapped) {
+          topmost = layer;
+          break;
+        }
+      }
+      if (topmost != NULL)
+        break;
+    }
+  }
+
+  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+    PhocSeat *seat = PHOC_SEAT (elem->data);
+
+    g_assert (PHOC_IS_SEAT (seat));
+    phoc_seat_set_focus_layer(seat, topmost ? topmost->layer_surface : NULL);
+  }
 }
 
 static void handle_output_destroy(struct wl_listener *listener, void *data) {
-	struct roots_layer_surface *layer =
+	PhocLayerSurface *layer =
 		wl_container_of(listener, layer, output_destroy);
 	layer->layer_surface->output = NULL;
 	wl_list_remove(&layer->output_destroy.link);
@@ -342,17 +345,27 @@ static void handle_output_destroy(struct wl_listener *listener, void *data) {
 
 static void handle_surface_commit(struct wl_listener *listener, void *data) {
 	PhocServer *server = phoc_server_get_default ();
-	struct roots_layer_surface *layer =
+	PhocLayerSurface *layer =
 		wl_container_of(listener, layer, surface_commit);
 	struct wlr_layer_surface_v1 *layer_surface = layer->layer_surface;
 	struct wlr_output *wlr_output = layer_surface->output;
 	if (wlr_output != NULL) {
 		PhocOutput *output = wlr_output->data;
 		struct wlr_box old_geo = layer->geo;
-		arrange_layers(output);
+
+		bool layer_changed = layer->layer != layer_surface->current.layer;
+		if (layer_changed) {
+			wl_list_remove(&layer->link);
+			wl_list_insert(&output->layers[layer_surface->current.layer],
+				&layer->link);
+			layer->layer = layer_surface->current.layer;
+		}
+
+		phoc_layer_shell_arrange (output);
+		phoc_layer_shell_update_focus ();
 
 		// Cursor changes which happen as a consequence of resizing a layer
-		// surface are applied in arrange_layers. Because the resize happens
+		// surface are applied in phoc_layer_shell_arrange. Because the resize happens
 		// before the underlying surface changes, it will only receive a cursor
 		// update if the new cursor position crosses the *old* sized surface in
 		// the *new* layer surface.
@@ -361,18 +374,11 @@ static void handle_surface_commit(struct wl_listener *listener, void *data) {
 		struct wlr_surface *surface = layer_surface->surface;
 		if (surface->previous.width != surface->current.width ||
 				surface->previous.height != surface->current.height) {
-			update_cursors(layer, &server->input->seats);
+			update_cursors(layer, phoc_input_get_seats (server->input));
 		}
 
 		bool geo_changed =
 			memcmp(&old_geo, &layer->geo, sizeof(struct wlr_box)) != 0;
-		bool layer_changed = layer->layer != layer_surface->current.layer;
-		if (layer_changed) {
-			wl_list_remove(&layer->link);
-			wl_list_insert(&output->layers[layer_surface->current.layer],
-				&layer->link);
-			layer->layer = layer_surface->current.layer;
-		}
 		if (geo_changed || layer_changed) {
 			phoc_output_damage_whole_local_surface(output, layer_surface->surface,
 							       old_geo.x, old_geo.y);
@@ -385,31 +391,11 @@ static void handle_surface_commit(struct wl_listener *listener, void *data) {
 	}
 }
 
-static void unmap(struct wlr_layer_surface_v1 *layer_surface) {
-	struct roots_layer_surface *layer = layer_surface->data;
-	struct wlr_output *wlr_output = layer_surface->output;
-	if (wlr_output != NULL) {
-		phoc_output_damage_whole_local_surface(wlr_output->data, layer_surface->surface,
-			layer->geo.x, layer->geo.y);
-	}
-}
 
 static void handle_destroy(struct wl_listener *listener, void *data) {
-	struct roots_layer_surface *layer = wl_container_of(
+	PhocLayerSurface *layer = wl_container_of(
 			listener, layer, destroy);
-	if (layer->layer_surface->mapped) {
-		unmap(layer->layer_surface);
-	}
-	wl_list_remove(&layer->link);
-	wl_list_remove(&layer->destroy.link);
-	wl_list_remove(&layer->map.link);
-	wl_list_remove(&layer->unmap.link);
-	wl_list_remove(&layer->surface_commit.link);
-	if (layer->layer_surface->output) {
-		wl_list_remove(&layer->output_destroy.link);
-		arrange_layers((PhocOutput *)layer->layer_surface->output->data);
-	}
-	free(layer);
+	g_object_unref (layer);
 }
 
 static void subsurface_destroy(struct roots_layer_subsurface *subsurface) {
@@ -424,7 +410,7 @@ static void subsurface_destroy(struct roots_layer_subsurface *subsurface) {
 static struct roots_layer_popup *popup_create(struct wlr_xdg_popup *wlr_popup);
 static struct roots_layer_subsurface *layer_subsurface_create(struct wlr_subsurface *wlr_subsurface);
 
-static struct roots_layer_surface *popup_get_root_layer(struct roots_layer_popup *popup) {
+static PhocLayerSurface *popup_get_root_layer(struct roots_layer_popup *popup) {
 	while (popup->parent_type == LAYER_PARENT_POPUP) {
 		popup = popup->parent_popup;
 	}
@@ -432,10 +418,10 @@ static struct roots_layer_surface *popup_get_root_layer(struct roots_layer_popup
 }
 
 static void popup_unconstrain(struct roots_layer_popup *popup) {
-	struct roots_layer_surface *layer = popup_get_root_layer(popup);
+	PhocLayerSurface *layer = popup_get_root_layer(popup);
 	struct wlr_xdg_popup *wlr_popup = popup->wlr_popup;
 
-	PhocOutput *output = layer->layer_surface->output->data;
+	PhocOutput *output = phoc_layer_surface_get_output (layer);
 
 	// the output box expressed in the coordinate system of the toplevel parent
 	// of the popup
@@ -455,7 +441,7 @@ static void popup_damage(struct roots_layer_popup *layer_popup, bool whole) {
 	int popup_sx = popup->geometry.x - popup->base->geometry.x;
 	int popup_sy = popup->geometry.y - popup->base->geometry.y;
 	int ox = popup_sx, oy = popup_sy;
-	struct roots_layer_surface *layer;
+	PhocLayerSurface *layer;
 	while (layer_popup->parent_type == LAYER_PARENT_POPUP) {
 		layer_popup = layer_popup->parent_popup;
 		ox += layer_popup->wlr_popup->geometry.x;
@@ -465,16 +451,15 @@ static void popup_damage(struct roots_layer_popup *layer_popup, bool whole) {
 	ox += layer->geo.x;
 	oy += layer->geo.y;
 
-	struct wlr_output *wlr_output = layer->layer_surface->output;
-	if (!wlr_output) {
+	PhocOutput *output = phoc_layer_surface_get_output (layer);
+	if (!output) {
 		return;
 	}
 
 	if (whole) {
-		phoc_output_damage_whole_local_surface(wlr_output->data, surface, ox, oy);
+		phoc_output_damage_whole_local_surface(output, surface, ox, oy);
 	} else {
-		phoc_output_damage_from_local_surface(layer->layer_surface->output->data,
-						      surface, ox, oy);
+		phoc_output_damage_from_local_surface(output, surface, ox, oy);
 	}
 }
 
@@ -501,14 +486,20 @@ static void popup_new_subsurface(struct wl_listener *listener, void *data) {
 static void popup_handle_map(struct wl_listener *listener, void *data) {
 	PhocServer *server = phoc_server_get_default ();
 	struct roots_layer_popup *popup = wl_container_of(listener, popup, map);
-	struct roots_layer_surface *layer = popup_get_root_layer(popup);
+	PhocLayerSurface *layer = popup_get_root_layer(popup);
 	struct wlr_output *wlr_output = layer->layer_surface->output;
 	if (!wlr_output) {
 		return;
 	}
 
 	struct wlr_subsurface *child;
-	wl_list_for_each(child, &popup->wlr_popup->base->surface->subsurfaces, parent_link) {
+	wl_list_for_each(child, &popup->wlr_popup->base->surface->subsurfaces_below, parent_link) {
+		struct roots_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+		new_subsurface->parent_type = LAYER_PARENT_POPUP;
+		new_subsurface->parent_popup = popup;
+		wl_list_insert(&popup->subsurfaces, &new_subsurface->link);
+	}
+	wl_list_for_each(child, &popup->wlr_popup->base->surface->subsurfaces_above, parent_link) {
 		struct roots_layer_subsurface *new_subsurface = layer_subsurface_create(child);
 		new_subsurface->parent_type = LAYER_PARENT_POPUP;
 		new_subsurface->parent_popup = popup;
@@ -575,7 +566,7 @@ static struct roots_layer_popup *popup_create(struct wlr_xdg_popup *wlr_popup) {
 }
 
 static void handle_new_popup(struct wl_listener *listener, void *data) {
-	struct roots_layer_surface *roots_layer_surface =
+	PhocLayerSurface *roots_layer_surface =
 		wl_container_of(listener, roots_layer_surface, new_popup);
 	struct wlr_xdg_popup *wlr_popup = data;
 	struct roots_layer_popup *popup = popup_create(wlr_popup);
@@ -584,7 +575,7 @@ static void handle_new_popup(struct wl_listener *listener, void *data) {
 	popup_unconstrain(popup);
 }
 
-static struct roots_layer_surface *subsurface_get_root_layer(struct roots_layer_subsurface *subsurface) {
+static PhocLayerSurface *subsurface_get_root_layer(struct roots_layer_subsurface *subsurface) {
 	while (subsurface->parent_type == LAYER_PARENT_SUBSURFACE) {
 		subsurface = subsurface->parent_subsurface;
 	}
@@ -595,19 +586,19 @@ static struct roots_layer_surface *subsurface_get_root_layer(struct roots_layer_
 }
 
 static void subsurface_damage(struct roots_layer_subsurface *subsurface, bool whole) {
-	struct roots_layer_surface *layer = subsurface_get_root_layer(subsurface);
-	struct wlr_output *wlr_output = layer->layer_surface->output;
-	if (!wlr_output) {
+	PhocLayerSurface *layer = subsurface_get_root_layer(subsurface);
+	PhocOutput *output = phoc_layer_surface_get_output (layer);
+	if (!output) {
 		return;
 	}
 	int ox = subsurface->wlr_subsurface->current.x + layer->geo.x;
 	int oy = subsurface->wlr_subsurface->current.y + layer->geo.y;
 	if (whole) {
-		phoc_output_damage_whole_local_surface(wlr_output->data,
+		phoc_output_damage_whole_local_surface(output,
 						       subsurface->wlr_subsurface->surface,
 						       ox, oy);
 	} else {
-		phoc_output_damage_from_local_surface(wlr_output->data,
+		phoc_output_damage_from_local_surface(output,
 						      subsurface->wlr_subsurface->surface,
 						      ox, oy);
 	}
@@ -629,7 +620,13 @@ static void subsurface_handle_map(struct wl_listener *listener, void *data) {
 	struct roots_layer_subsurface *subsurface = wl_container_of(listener, subsurface, map);
 
 	struct wlr_subsurface *child;
-	wl_list_for_each(child, &subsurface->wlr_subsurface->surface->subsurfaces, parent_link) {
+	wl_list_for_each(child, &subsurface->wlr_subsurface->surface->subsurfaces_below, parent_link) {
+		struct roots_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+		new_subsurface->parent_type = LAYER_PARENT_SUBSURFACE;
+		new_subsurface->parent_subsurface = subsurface;
+		wl_list_insert(&subsurface->subsurfaces, &new_subsurface->link);
+	}
+	wl_list_for_each(child, &subsurface->wlr_subsurface->surface->subsurfaces_above, parent_link) {
 		struct roots_layer_subsurface *new_subsurface = layer_subsurface_create(child);
 		new_subsurface->parent_type = LAYER_PARENT_SUBSURFACE;
 		new_subsurface->parent_subsurface = subsurface;
@@ -691,7 +688,7 @@ static struct roots_layer_subsurface *layer_subsurface_create(struct wlr_subsurf
 }
 
 static void handle_new_subsurface(struct wl_listener *listener, void *data) {
-	struct roots_layer_surface *roots_layer_surface =
+	PhocLayerSurface *roots_layer_surface =
 		wl_container_of(listener, roots_layer_surface, new_subsurface);
 	struct wlr_subsurface *wlr_subsurface = data;
 
@@ -703,14 +700,20 @@ static void handle_new_subsurface(struct wl_listener *listener, void *data) {
 
 static void handle_map(struct wl_listener *listener, void *data) {
 	struct wlr_layer_surface_v1 *layer_surface = data;
-	struct roots_layer_surface *layer = layer_surface->data;
-	struct wlr_output *wlr_output = layer_surface->output;
-	if (!wlr_output) {
+	PhocLayerSurface *layer = PHOC_LAYER_SURFACE (layer_surface->data);
+	PhocOutput *output = phoc_layer_surface_get_output (layer);
+	if (!output) {
 		return;
 	}
 
 	struct wlr_subsurface *subsurface;
-	wl_list_for_each(subsurface, &layer_surface->surface->subsurfaces, parent_link) {
+	wl_list_for_each(subsurface, &layer_surface->surface->subsurfaces_below, parent_link) {
+		struct roots_layer_subsurface *roots_subsurface = layer_subsurface_create(subsurface);
+		roots_subsurface->parent_type = LAYER_PARENT_LAYER;
+		roots_subsurface->parent_layer = layer;
+		wl_list_insert(&layer->subsurfaces, &roots_subsurface->link);
+	}
+	wl_list_for_each(subsurface, &layer_surface->surface->subsurfaces_above, parent_link) {
 		struct roots_layer_subsurface *roots_subsurface = layer_subsurface_create(subsurface);
 		roots_subsurface->parent_type = LAYER_PARENT_LAYER;
 		roots_subsurface->parent_layer = layer;
@@ -720,15 +723,15 @@ static void handle_map(struct wl_listener *listener, void *data) {
 	layer->new_subsurface.notify = handle_new_subsurface;
 	wl_signal_add(&layer_surface->surface->events.new_subsurface, &layer->new_subsurface);
 
-	phoc_output_damage_whole_local_surface(wlr_output->data,
+	phoc_output_damage_whole_local_surface(output,
 					       layer_surface->surface, layer->geo.x,
 					       layer->geo.y);
-	wlr_surface_send_enter(layer_surface->surface, wlr_output);
+	wlr_surface_send_enter(layer_surface->surface, output->wlr_output);
 }
 
 static void handle_unmap(struct wl_listener *listener, void *data) {
 	PhocServer *server = phoc_server_get_default ();
-	struct roots_layer_surface *layer = wl_container_of(
+	PhocLayerSurface *layer = wl_container_of(
 			listener, layer, unmap);
 
 	struct roots_layer_subsurface *subsurface, *tmp;
@@ -737,7 +740,7 @@ static void handle_unmap(struct wl_listener *listener, void *data) {
 	}
 	wl_list_remove(&layer->new_subsurface.link);
 
-	unmap(layer->layer_surface);
+	phoc_layer_surface_unmap (layer);
 	phoc_input_update_cursor_focus(server->input);
 }
 
@@ -746,7 +749,7 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 	struct wlr_layer_surface_v1 *layer_surface = data;
 	PhocDesktop *desktop =
 		wl_container_of(listener, desktop, layer_shell_surface);
-	wlr_log(WLR_DEBUG, "new layer surface: namespace %s layer %d anchor %d "
+	g_debug ("new layer surface: namespace %s layer %d anchor %d "
 			"size %dx%d margin %d,%d,%d,%d",
 		layer_surface->namespace, layer_surface->client_pending.layer,
 		layer_surface->client_pending.anchor,
@@ -759,17 +762,17 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 
 	if (!layer_surface->output) {
 		PhocInput *input = server->input;
-		struct roots_seat *seat = input_last_active_seat(input);
+		PhocSeat *seat = phoc_input_get_last_active_seat(input);
 		assert(seat); // Technically speaking we should handle this case
-		struct roots_cursor *cursor = roots_seat_get_cursor(seat);
+		PhocCursor *cursor = phoc_seat_get_cursor(seat);
 		struct wlr_output *output =
 			wlr_output_layout_output_at(desktop->layout,
 					cursor->cursor->x,
 					cursor->cursor->y);
 		if (!output) {
-			wlr_log(WLR_ERROR, "Couldn't find output at (%.0f,%.0f)",
-				cursor->cursor->x,
-				cursor->cursor->y);
+			g_critical ("Couldn't find output at (%.0f,%.0f)",
+                                    cursor->cursor->x,
+                                    cursor->cursor->y);
 			output = wlr_output_layout_get_center_output(desktop->layout);
 		}
 		if (output) {
@@ -780,8 +783,7 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 		}
 	}
 
-	struct roots_layer_surface *roots_surface =
-		calloc(1, sizeof(struct roots_layer_surface));
+	PhocLayerSurface *roots_surface = phoc_layer_surface_new ();
 	if (!roots_surface) {
 		return;
 	}
@@ -816,7 +818,8 @@ void handle_layer_shell_surface(struct wl_listener *listener, void *data) {
 	struct wlr_layer_surface_v1_state old_state = layer_surface->current;
 	layer_surface->current = layer_surface->client_pending;
 
-	arrange_layers(output);
+	phoc_layer_shell_arrange (output);
+	phoc_layer_shell_update_focus ();
 
 	layer_surface->current = old_state;
 }
