@@ -1,10 +1,13 @@
-#ifndef ROOTSTON_LAYERS_H
-#define ROOTSTON_LAYERS_H
+#pragma once
+
 #include <stdbool.h>
 #include <wlr/types/wlr_box.h>
 #include <wlr/types/wlr_surface.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
 #include "output.h"
+#include "layer-surface.h"
+
+G_BEGIN_DECLS
 
 enum layer_parent {
 	LAYER_PARENT_LAYER,
@@ -12,27 +15,10 @@ enum layer_parent {
 	LAYER_PARENT_SUBSURFACE
 };
 
-struct roots_layer_surface {
-	struct wlr_layer_surface_v1 *layer_surface;
-	struct wl_list link;
-
-	struct wl_listener destroy;
-	struct wl_listener map;
-	struct wl_listener unmap;
-	struct wl_listener surface_commit;
-	struct wl_listener output_destroy;
-	struct wl_listener new_popup;
-	struct wl_listener new_subsurface;
-	struct wl_list subsurfaces; // roots_layer_subsurface::link
-
-	struct wlr_box geo;
-	enum zwlr_layer_shell_v1_layer layer;
-};
-
 struct roots_layer_popup {
 	enum layer_parent parent_type;
 	union {
-		struct roots_layer_surface *parent_layer;
+		PhocLayerSurface *parent_layer;
 		struct roots_layer_popup *parent_popup;
 	};
 
@@ -49,7 +35,7 @@ struct roots_layer_popup {
 struct roots_layer_subsurface {
 	enum layer_parent parent_type;
 	union {
-		struct roots_layer_surface *parent_layer;
+		PhocLayerSurface *parent_layer;
 		struct roots_layer_popup *parent_popup;
 		struct roots_layer_subsurface *parent_subsurface;
 	};
@@ -64,6 +50,7 @@ struct roots_layer_subsurface {
 	struct wl_list subsurfaces; // roots_layer_subsurface::link
 };
 
-void arrange_layers(PhocOutput *output);
+void phoc_layer_shell_arrange (PhocOutput *output);
+void phoc_layer_shell_update_focus (void);
 
-#endif
+G_END_DECLS
