@@ -1,9 +1,26 @@
+/*
+ * Copyright (C) 2021 Purism SPC
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ *
+ * Authors: The wlroots authors
+ *          Sebastian Krzyszkowiak
+ *          Guido Günther <agx@sigxcpu.org>
+ */
+
 #define G_LOG_DOMAIN "phoc-render"
 
 #include "config.h"
+#include "layers.h"
+#include "seat.h"
+#include "server.h"
+#include "render.h"
+#include "xwayland-surface.h"
 
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
+#include <drm_fourcc.h>
+#include <fcntl.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <time.h>
@@ -11,21 +28,24 @@
 #include <wlr/config.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/render/gles2.h>
+#include <wlr/render/egl.h>
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_matrix.h>
 #include <wlr/types/wlr_buffer.h>
 #include <wlr/types/wlr_linux_dmabuf_v1.h>
-#include <wlr/util/log.h>
 #include <wlr/util/region.h>
 #include <wlr/version.h>
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
-#include "layers.h"
-#include "render.h"
-#include "server.h"
-#include "render.h"
 
-#define TOUCH_POINT_RADIUS 30
+/* Private wlr_allocator headers */
+struct wlr_gbm_allocator *wlr_gbm_allocator_create(int drm_fd);
+struct wlr_buffer *wlr_allocator_create_buffer (struct wlr_allocator *alloc, int width, int height, const struct wlr_drm_format *format);
+void wlr_allocator_destroy (struct wlr_allocator *alloc);
+struct wlr_drm_format *wlr_drm_format_create (uint32_t format);
+/* ----------------------------- */
+
+#define TOUCH_POINT_SIZE 20
 #define TOUCH_POINT_BORDER 0.1
 
 #define COLOR_BLACK                {0.0f, 0.0f, 0.0f, 1.0f}
@@ -34,9 +54,45 @@
 #define COLOR_TRANSPARENT_YELLOW   {0.5f, 0.5f, 0.0f, 0.5f}
 #define COLOR_TRANSPARENT_MAGENTA  {0.5f, 0.0f, 0.5f, 0.5f}
 
+
+/**
+ * PhocRenderer:
+ *
+ * The renderer
+ */
+
+enum {
+  RENDER_START,
+  RENDER_END,
+  N_SIGNALS
+};
+static guint signals[N_SIGNALS] = { 0 };
+
+enum {
+  PROP_0,
+  PROP_WLR_RENDERER,
+  PROP_LAST_PROP
+};
+static GParamSpec *props[PROP_LAST_PROP];
+
+struct _PhocRenderer {
+  GObject               parent;
+
+  struct wlr_renderer  *wlr_renderer;
+  struct wlr_allocator *allocator;
+};
+G_DEFINE_TYPE (PhocRenderer, phoc_renderer, G_TYPE_OBJECT)
+
+
 struct render_data {
-	pixman_region32_t *damage;
-	float alpha;
+  pixman_region32_t *damage;
+  float alpha;
+};
+
+struct view_render_data {
+  PhocView *view;
+  int width;
+  int height;
 };
 
 struct touch_point_data {
@@ -45,6 +101,45 @@ struct touch_point_data {
   double y;
 };
 
+
+static void
+phoc_renderer_set_property (GObject      *object,
+                            guint         property_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
+{
+  PhocRenderer *self = PHOC_RENDERER (object);
+
+  switch (property_id) {
+  case PROP_WLR_RENDERER:
+    self->wlr_renderer = g_value_get_pointer (value);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
+  }
+}
+
+
+static void
+phoc_renderer_get_property (GObject    *object,
+                            guint       property_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
+{
+  PhocRenderer *self = PHOC_RENDERER (object);
+
+  switch (property_id) {
+  case PROP_WLR_RENDERER:
+    g_value_set_pointer (value, self->wlr_renderer);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
+  }
+}
+
+
 static void scissor_output(struct wlr_output *wlr_output,
 		pixman_box32_t *rect) {
 	struct wlr_renderer *renderer =
@@ -70,14 +165,15 @@ static void scissor_output(struct wlr_output *wlr_output,
 
 static void render_texture(struct wlr_output *wlr_output,
 		pixman_region32_t *output_damage, struct wlr_texture *texture,
-		const struct wlr_box *box, const float matrix[static 9],
+		const struct wlr_fbox *src_box, const struct wlr_box *dst_box,
+		const float matrix[static 9],
 		float rotation, float alpha) {
 	struct wlr_renderer *renderer =
 		wlr_backend_get_renderer(wlr_output->backend);
 	assert(renderer);
 
 	struct wlr_box rotated;
-	wlr_box_rotated_bounds(&rotated, box, rotation);
+	wlr_box_rotated_bounds(&rotated, dst_box, rotation);
 
 	pixman_region32_t damage;
 	pixman_region32_init(&damage);
@@ -93,7 +189,12 @@ static void render_texture(struct wlr_output *wlr_output,
 	pixman_box32_t *rects = pixman_region32_rectangles(&damage, &nrects);
 	for (int i = 0; i < nrects; ++i) {
 		scissor_output(wlr_output, &rects[i]);
-		wlr_render_texture_with_matrix(renderer, texture, matrix, alpha);
+
+		if (src_box != NULL) {
+			wlr_render_subtexture_with_matrix(renderer, texture, src_box, matrix, alpha);
+		} else {
+			wlr_render_texture_with_matrix(renderer, texture, matrix, alpha);
+		}
 	}
 
 buffer_damage_finish:
@@ -108,8 +209,10 @@ collect_touch_points (PhocOutput *output, struct wlr_surface *surface, struct wl
     return;
   }
 
-  struct roots_seat *seat;
-  wl_list_for_each(seat, &server->input->seats, link) {
+  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+    PhocSeat *seat = PHOC_SEAT (elem->data);
+
+    g_assert (PHOC_IS_SEAT (seat));
     struct wlr_touch_point *point;
     wl_list_for_each(point, &seat->seat->touch_state.touch_points, link) {
       if (point->surface != surface) { continue; }
@@ -135,28 +238,31 @@ static void render_surface_iterator(PhocOutput *output,
 		return;
 	}
 
-	struct wlr_box box = *_box;
-	phoc_output_scale_box(wlr_output->data, &box, scale);
-	phoc_output_scale_box(wlr_output->data, &box, wlr_output->scale);
+	struct wlr_fbox src_box;
+	wlr_surface_get_buffer_source_box(surface, &src_box);
+
+	struct wlr_box dst_box = *_box;
+	phoc_output_scale_box(wlr_output->data, &dst_box, scale);
+	phoc_output_scale_box(wlr_output->data, &dst_box, wlr_output->scale);
 
 	float matrix[9];
 	enum wl_output_transform transform =
 		wlr_output_transform_invert(surface->current.transform);
-	wlr_matrix_project_box(matrix, &box, transform, rotation,
+	wlr_matrix_project_box(matrix, &dst_box, transform, rotation,
 		wlr_output->transform_matrix);
 
 	render_texture(wlr_output, output_damage,
-		texture, &box, matrix, rotation, alpha);
+		texture, &src_box, &dst_box, matrix, rotation, alpha);
 
 	wlr_presentation_surface_sampled_on_output(output->desktop->presentation,
 		surface, wlr_output);
 
-	collect_touch_points(output, surface, box, scale);
+	collect_touch_points(output, surface, dst_box, scale);
 }
 
 static void render_decorations(PhocOutput *output,
-		struct roots_view *view, struct render_data *data) {
-	if (!view->decorated || view->wlr_surface == NULL) {
+		PhocView *view, struct render_data *data) {
+	if (!view->decorated || !phoc_view_is_mapped (view)) {
 		return;
 	}
 
@@ -194,15 +300,15 @@ buffer_damage_finish:
 	pixman_region32_fini(&damage);
 }
 
-static void render_view(PhocOutput *output, struct roots_view *view,
+static void render_view(PhocOutput *output, PhocView *view,
 		struct render_data *data) {
 	// Do not render views fullscreened on other outputs
-	if (view->fullscreen_output != NULL && view->fullscreen_output != output) {
+	if (view_is_fullscreen (view) && view->fullscreen_output != output) {
 		return;
 	}
 
 	data->alpha = view->alpha;
-	if (view->fullscreen_output == NULL) {
+	if (!view_is_fullscreen (view)) {
 		render_decorations(output, view, data);
 	}
 	phoc_output_view_for_each_surface(output, view, render_surface_iterator, data);
@@ -229,9 +335,11 @@ static bool scan_out_fullscreen_view(PhocOutput *output) {
 	struct wlr_output *wlr_output = output->wlr_output;
 	PhocServer *server = phoc_server_get_default ();
 
-	struct roots_seat *seat;
-	wl_list_for_each(seat, &server->input->seats, link) {
-		struct roots_drag_icon *drag_icon = seat->drag_icon;
+	for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+		PhocSeat *seat = PHOC_SEAT (elem->data);
+
+		g_assert (PHOC_IS_SEAT (seat));
+		PhocDragIcon *drag_icon = seat->drag_icon;
 		if (drag_icon && drag_icon->wlr_drag_icon->mapped) {
 			return false;
 		}
@@ -249,9 +357,9 @@ static bool scan_out_fullscreen_view(PhocOutput *output) {
 		}
 	}
 
-	struct roots_view *view = output->fullscreen_view;
+	PhocView *view = output->fullscreen_view;
 	assert(view != NULL);
-	if (view->wlr_surface == NULL) {
+	if (!phoc_view_is_mapped (view)) {
 		return false;
 	}
 	size_t n_surfaces = 0;
@@ -262,9 +370,9 @@ static bool scan_out_fullscreen_view(PhocOutput *output) {
 	}
 
 #if WLR_HAS_XWAYLAND
-	if (view->type == ROOTS_XWAYLAND_VIEW) {
-		struct roots_xwayland_surface *xwayland_surface =
-			roots_xwayland_surface_from_view(view);
+	if (view->type == PHOC_XWAYLAND_VIEW) {
+		PhocXWaylandSurface *xwayland_surface =
+			phoc_xwayland_surface_from_view(view);
 		if (!wl_list_empty(&xwayland_surface->xwayland_surface->children)) {
 			return false;
 		}
@@ -282,16 +390,10 @@ static bool scan_out_fullscreen_view(PhocOutput *output) {
 		return false;
 	}
 
-#if WLR_VERSION_MAJOR == 0 && WLR_VERSION_MINOR < 11
-	if (!wlr_output_attach_buffer(wlr_output, surface->buffer)) {
-		return false;
-	}
-#else
 	wlr_output_attach_buffer(wlr_output, &surface->buffer->base);
 	if (!wlr_output_test(wlr_output)) {
 		return false;
 	}
-#endif
 
 	wlr_presentation_surface_sampled_on_output(output->desktop->presentation, surface, output->wlr_output);
 
@@ -334,13 +436,13 @@ color_hsv_to_rgb (float* color)
 }
 
 static struct wlr_box
-wlr_box_from_touch_point (struct touch_point_data *touch_point, int radius)
+wlr_box_from_touch_point (struct touch_point_data *touch_point, int width, int height)
 {
   return (struct wlr_box) {
-    .x = touch_point->x - radius / 2.0,
-    .y = touch_point->y - radius / 2.0,
-    .width = radius,
-    .height = radius
+    .x = touch_point->x - width / 2.0,
+    .y = touch_point->y - height / 2.0,
+    .width = width,
+    .height = height
   };
 }
 
@@ -352,16 +454,23 @@ render_touch_point_cb (gpointer data, gpointer user_data)
   PhocOutput *output = user_data;
   struct wlr_output *wlr_output = output->wlr_output;
 
-  struct wlr_box point_box = wlr_box_from_touch_point (touch_point, TOUCH_POINT_RADIUS * wlr_output->scale);
+  int size = TOUCH_POINT_SIZE * wlr_output->scale;
+  struct wlr_box point_box = wlr_box_from_touch_point (touch_point, size, size);
 
   struct wlr_renderer *renderer = wlr_backend_get_renderer (wlr_output->backend);
 
   float color[4] = {touch_point->id * 100 + 240, 1.0, 1.0, 0.75};
   color_hsv_to_rgb (color);
-  wlr_render_ellipse (renderer, &point_box, color, wlr_output->transform_matrix);
+  wlr_render_rect (renderer, &point_box, color, wlr_output->transform_matrix);
 
-  point_box = wlr_box_from_touch_point (touch_point, TOUCH_POINT_RADIUS * (1.0 - TOUCH_POINT_BORDER) * wlr_output->scale);
-  wlr_render_ellipse(renderer, &point_box, (float[])COLOR_TRANSPARENT_WHITE, wlr_output->transform_matrix);
+  size = TOUCH_POINT_SIZE * (1.0 - TOUCH_POINT_BORDER) * wlr_output->scale;
+  point_box = wlr_box_from_touch_point (touch_point, size, size);
+  wlr_render_rect (renderer, &point_box, (float[])COLOR_TRANSPARENT_WHITE, wlr_output->transform_matrix);
+
+  point_box = wlr_box_from_touch_point (touch_point, 8 * wlr_output->scale, 2 * wlr_output->scale);
+  wlr_render_rect (renderer, &point_box, color, wlr_output->transform_matrix);
+  point_box = wlr_box_from_touch_point (touch_point, 2 * wlr_output->scale, 8 * wlr_output->scale);
+  wlr_render_rect (renderer, &point_box, color, wlr_output->transform_matrix);
 }
 
 static void
@@ -382,7 +491,8 @@ damage_touch_point_cb (gpointer data, gpointer user_data)
   PhocOutput *output = user_data;
   struct wlr_output *wlr_output = output->wlr_output;
 
-  struct wlr_box box = wlr_box_from_touch_point (touch_point, TOUCH_POINT_RADIUS * wlr_output->scale);
+  int size = TOUCH_POINT_SIZE * wlr_output->scale;
+  struct wlr_box box = wlr_box_from_touch_point (touch_point, size, size);
   pixman_region32_t region;
   pixman_region32_init_rect(&region, box.x, box.y, box.width, box.height);
   wlr_output_damage_add(output->damage, &region);
@@ -392,24 +502,26 @@ damage_touch_point_cb (gpointer data, gpointer user_data)
 static void
 damage_touch_points (PhocOutput *output)
 {
-  if (!g_list_length (output->debug_touch_points)) {
+  if (output->debug_touch_points == NULL)
     return;
-  }
+
   g_list_foreach (output->debug_touch_points, damage_touch_point_cb, output);
   wlr_output_schedule_frame(output->wlr_output);
 }
 
 static void
-view_render_iterator (struct wlr_surface *surface, int sx, int sy, void *data)
+view_render_iterator (struct wlr_surface *surface, int sx, int sy, void *_data)
 {
   if (!wlr_surface_has_buffer (surface)) {
     return;
   }
 
   PhocServer *server = phoc_server_get_default ();
+  PhocRenderer *self = phoc_server_get_renderer (server);
   struct wlr_texture *view_texture = wlr_surface_get_texture (surface);
 
-  struct roots_view *view = data;
+  struct view_render_data *data = _data;
+  PhocView *view = data->view;
   struct wlr_surface *root = view->wlr_surface;
 
   struct wlr_box box;
@@ -421,57 +533,56 @@ view_render_iterator (struct wlr_surface *surface, int sx, int sy, void *data)
   float mat[16];
   wlr_matrix_identity (mat);
 
-  // NDC
-  wlr_matrix_translate (mat, -1, -1);
-  wlr_matrix_scale (mat, 2, 2);
-
-  wlr_matrix_scale (mat, 1 / (float)box.width, 1 / (float)box.height);
-  wlr_matrix_translate (mat, -geo.x, -geo.y);
-
+  wlr_matrix_scale (mat, data->width / (float)geo.width, data->height / (float)geo.height);
   wlr_matrix_scale (mat, 1 / (float)root->current.scale, 1 / (float)root->current.scale);
   wlr_matrix_scale (mat, view->scale, view->scale);
   wlr_matrix_scale (mat, root->current.scale / surface->current.scale, root->current.scale / surface->current.scale);
+  wlr_matrix_translate (mat, -geo.x, -geo.y);
 
-  wlr_render_texture (server->renderer, view_texture, mat, sx * surface->current.scale, sy * surface->current.scale, 1.0);
+  wlr_render_texture (self->wlr_renderer,
+                      view_texture, mat, sx * surface->current.scale,
+                      sy * surface->current.scale,
+                      1.0);
 }
 
 gboolean
-view_render_to_buffer (struct roots_view *view, int width, int height, int stride, uint32_t *flags, void* data)
+phoc_renderer_render_view_to_buffer (PhocRenderer *self,
+                                     PhocView     *view,
+                                     int           width,
+                                     int           height,
+                                     int           stride,
+                                     uint32_t     *flags,
+                                     void         *data)
 {
-  PhocServer *server = phoc_server_get_default ();
   struct wlr_surface *surface = view->wlr_surface;
-  struct wlr_egl *egl = wlr_gles2_renderer_get_egl (server->renderer);
-  GLuint tex, fbo;
 
-  if (!surface || !wlr_egl_make_current (egl, EGL_NO_SURFACE, NULL)) {
-    return FALSE;
+  g_return_val_if_fail (surface, false);
+  g_return_val_if_fail (self->allocator, false);
+
+  struct wlr_drm_format *fmt = wlr_drm_format_create (DRM_FORMAT_ARGB8888);
+  struct wlr_buffer *buffer = wlr_allocator_create_buffer (self->allocator, width, height, fmt);
+
+  if (!buffer) {
+    free (fmt);
+    g_return_val_if_reached (false);
   }
 
-  glGenTextures (1, &tex);
-  glBindTexture (GL_TEXTURE_2D, tex);
-  glTexImage2D (GL_TEXTURE_2D, 0, GL_BGRA_EXT, width, height, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, NULL);
-  glBindTexture (GL_TEXTURE_2D, 0);
+  struct view_render_data render_data ={
+    .view = view,
+    .width = width,
+    .height = height
+  };
 
-  glGenFramebuffers (1, &fbo);
-  glBindFramebuffer (GL_FRAMEBUFFER, fbo);
-  glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, tex, 0);
+  wlr_renderer_begin_with_buffer (self->wlr_renderer, buffer);
+  wlr_renderer_clear (self->wlr_renderer, (float[])COLOR_TRANSPARENT);
+  wlr_surface_for_each_surface (surface, view_render_iterator, &render_data);
+  wlr_renderer_read_pixels (self->wlr_renderer, DRM_FORMAT_ARGB8888, NULL, stride, width, height, 0, 0, 0, 0, data);
+  wlr_renderer_end (self->wlr_renderer);
 
-  wlr_renderer_begin (server->renderer, width, height);
-  wlr_renderer_clear (server->renderer, (float[])COLOR_TRANSPARENT);
-  wlr_surface_for_each_surface (surface, view_render_iterator, view);
-  wlr_renderer_end (server->renderer);
+  wlr_buffer_drop (buffer);
+  free(fmt);
 
-  wlr_renderer_read_pixels (server->renderer, WL_SHM_FORMAT_ARGB8888, flags, stride, width, height, 0, 0, 0, 0, data);
-
-  glDeleteFramebuffers (1, &fbo);
-  glDeleteTextures (1, &tex);
-  glBindFramebuffer (GL_FRAMEBUFFER, 0);
-
-#if WLR_VERSION_MAJOR > 0 || WLR_VERSION_MINOR >= 11
-  wlr_egl_unset_current (egl);
-#endif
-
-  return TRUE;
+  return true;
 }
 
 static void surface_send_frame_done_iterator(PhocOutput *output,
@@ -481,13 +592,53 @@ static void surface_send_frame_done_iterator(PhocOutput *output,
 	wlr_surface_send_frame_done(surface, when);
 }
 
-void output_render(PhocOutput *output) {
+
+static void
+render_damage (PhocRenderer *self, PhocOutput *output)
+{
+  int nrects;
+  pixman_box32_t *rects;
+  struct wlr_box box;
+  pixman_region32_t previous_damage;
+
+  pixman_region32_init(&previous_damage);
+  pixman_region32_subtract(&previous_damage,
+                           &output->damage->previous[output->damage->previous_idx],
+                           &output->damage->current);
+
+  rects = pixman_region32_rectangles(&previous_damage, &nrects);
+  for (int i = 0; i < nrects; ++i) {
+    wlr_box_from_pixman_box32(&box, rects[i]);
+    wlr_render_rect(self->wlr_renderer, &box, (float[])COLOR_TRANSPARENT_MAGENTA,
+                    output->wlr_output->transform_matrix);
+  }
+
+  rects = pixman_region32_rectangles(&output->damage->current, &nrects);
+  for (int i = 0; i < nrects; ++i) {
+    wlr_box_from_pixman_box32(&box, rects[i]);
+    wlr_render_rect(self->wlr_renderer, &box, (float[])COLOR_TRANSPARENT_YELLOW,
+                    output->wlr_output->transform_matrix);
+  }
+  wlr_output_schedule_frame(output->wlr_output);
+  pixman_region32_fini(&previous_damage);
+}
+
+
+/**
+ * phoc_renderer_render_output:
+ * @self: The renderer
+ * @output: The output to render
+ *
+ * Render a given output.
+ */
+void phoc_renderer_render_output (PhocRenderer *self, PhocOutput *output) {
 	struct wlr_output *wlr_output = output->wlr_output;
-	PhocDesktop *desktop = output->desktop;
+	PhocDesktop *desktop = PHOC_DESKTOP (output->desktop);
 	PhocServer *server = phoc_server_get_default ();
-	struct wlr_renderer *renderer =
-		wlr_backend_get_renderer(wlr_output->backend);
-	assert(renderer);
+	struct wlr_renderer *wlr_renderer;
+
+        g_assert (PHOC_IS_RENDERER (self));
+        wlr_renderer = self->wlr_renderer;
 
 	if (!wlr_output->enabled) {
 		return;
@@ -498,35 +649,17 @@ void output_render(PhocOutput *output) {
 
 	float clear_color[] = COLOR_BLACK;
 
-	const struct wlr_box *output_box =
-		wlr_output_layout_get_box(desktop->layout, wlr_output);
+	g_signal_emit (self, signals[RENDER_START], 0, output);
 
 	// Check if we can delegate the fullscreen surface to the output
-	if (output->fullscreen_view != NULL &&
-			output->fullscreen_view->wlr_surface != NULL) {
-		struct roots_view *view = output->fullscreen_view;
-
-		// Make sure the view is centered on screen
-		struct wlr_box view_box;
-		view_get_box(view, &view_box);
-		double view_x = (double)(output_box->width - view_box.width) / 2 +
-			output_box->x;
-		double view_y = (double)(output_box->height - view_box.height) / 2 +
-			output_box->y;
-		view_move(view, view_x, view_y);
-
-		// Fullscreen views are rendered on a black background
-		clear_color[0] = clear_color[1] = clear_color[2] = 0;
-
-		// Check if we can scan-out the fullscreen view
+	if (phoc_output_has_fullscreen_view (output)) {
 		static bool last_scanned_out = false;
 		bool scanned_out = scan_out_fullscreen_view(output);
 
 		if (scanned_out && !last_scanned_out) {
-			wlr_log(WLR_DEBUG, "Scanning out fullscreen view");
-		}
-		if (last_scanned_out && !scanned_out) {
-			wlr_log(WLR_DEBUG, "Stopping fullscreen view scan out");
+			g_debug ("Starting fullscreen view scan out");
+		} else if (!scanned_out && last_scanned_out) {
+			g_debug ("Stopping fullscreen view scan out");
 		}
 		last_scanned_out = scanned_out;
 
@@ -566,7 +699,7 @@ void output_render(PhocOutput *output) {
 		goto buffer_damage_finish;
 	}
 
-	wlr_renderer_begin(renderer, wlr_output->width, wlr_output->height);
+	wlr_renderer_begin(wlr_renderer, wlr_output->width, wlr_output->height);
 
 	if (!pixman_region32_not_empty(&buffer_damage)) {
 		// Output isn't damaged but needs buffer swap
@@ -577,12 +710,12 @@ void output_render(PhocOutput *output) {
 	pixman_box32_t *rects = pixman_region32_rectangles(&buffer_damage, &nrects);
 	for (int i = 0; i < nrects; ++i) {
 		scissor_output(output->wlr_output, &rects[i]);
-		wlr_renderer_clear(renderer, clear_color);
+		wlr_renderer_clear(wlr_renderer, clear_color);
 	}
 
 	// If a view is fullscreen on this output, render it
 	if (output->fullscreen_view != NULL) {
-		struct roots_view *view = output->fullscreen_view;
+		PhocView *view = output->fullscreen_view;
 
 		render_view(output, view, &data);
 
@@ -590,9 +723,9 @@ void output_render(PhocOutput *output) {
 		// because all windows are rendered. Here we only want to render
 		// the fullscreen window's children so we have to traverse the tree.
 #ifdef PHOC_XWAYLAND
-		if (view->type == ROOTS_XWAYLAND_VIEW) {
-			struct roots_xwayland_surface *xwayland_surface =
-				roots_xwayland_surface_from_view(view);
+		if (view->type == PHOC_XWAYLAND_VIEW) {
+			PhocXWaylandSurface *xwayland_surface =
+				phoc_xwayland_surface_from_view(view);
 			phoc_output_xwayland_children_for_each_surface(output,
 				xwayland_surface->xwayland_surface,
 				render_surface_iterator, &data);
@@ -611,7 +744,7 @@ void output_render(PhocOutput *output) {
 		render_layer(output, &buffer_damage,
 			&output->layers[ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM]);
 
-		struct roots_view *view;
+		PhocView *view;
 			// Render all views
 		wl_list_for_each_reverse(view, &desktop->views, link) {
 			if (phoc_desktop_view_is_visible(desktop, view)) {
@@ -631,9 +764,14 @@ void output_render(PhocOutput *output) {
 
 renderer_end:
 	wlr_output_render_software_cursors(wlr_output, &buffer_damage);
-	wlr_renderer_scissor(renderer, NULL);
+	wlr_renderer_scissor(wlr_renderer, NULL);
 
 	render_touch_points (output);
+	g_signal_emit (self, signals[RENDER_END], 0, output);
+	if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_DAMAGE_TRACKING))
+		render_damage (self, output);
+
+	wlr_renderer_end(wlr_renderer);
 
 	int width, height;
 	wlr_output_transformed_resolution(wlr_output, &width, &height);
@@ -644,46 +782,106 @@ renderer_end:
 	wlr_region_transform(&frame_damage, &output->damage->current,
 		transform, width, height);
 
-	if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_DAMAGE_TRACKING)) {
-		pixman_region32_t previous_damage;
-		pixman_region32_init(&previous_damage);
-		pixman_region32_subtract(&previous_damage,
-			&output->damage->previous[output->damage->previous_idx], &output->damage->current);
-
-		struct wlr_box box;
-		rects = pixman_region32_rectangles(&previous_damage, &nrects);
-		for (int i = 0; i < nrects; ++i) {
-			wlr_box_from_pixman_box32(&box, rects[i]);
-			wlr_render_rect(renderer, &box, (float[])COLOR_TRANSPARENT_MAGENTA, wlr_output->transform_matrix);
-		}
-
-		rects = pixman_region32_rectangles(&output->damage->current, &nrects);
-		for (int i = 0; i < nrects; ++i) {
-			wlr_box_from_pixman_box32(&box, rects[i]);
-			wlr_render_rect(renderer, &box, (float[])COLOR_TRANSPARENT_YELLOW, wlr_output->transform_matrix);
-		}
-		wlr_output_schedule_frame(output->wlr_output);
-		pixman_region32_fini(&previous_damage);
-	}
-
-	wlr_renderer_end(renderer);
-
 	wlr_output_set_damage(wlr_output, &frame_damage);
 	pixman_region32_fini(&frame_damage);
 
 	if (!wlr_output_commit(wlr_output)) {
 		goto buffer_damage_finish;
 	}
-	output->last_frame = desktop->last_frame = now;
 
 buffer_damage_finish:
 	pixman_region32_fini(&buffer_damage);
 
 send_frame_done:
-	// Send frame done events to all surfaces
+	// Send frame done events to all visible surfaces
 	phoc_output_for_each_surface(output, surface_send_frame_done_iterator, &now, true);
 
 	damage_touch_points(output);
-	g_list_free_full(output->debug_touch_points, g_free);
-	output->debug_touch_points = NULL;
+	g_clear_list (&output->debug_touch_points, g_free);
+}
+
+
+static void
+phoc_renderer_constructed (GObject *object)
+{
+  PhocRenderer *self = PHOC_RENDERER (object);
+  int drm_fd = fcntl (wlr_renderer_get_drm_fd (self->wlr_renderer), F_DUPFD_CLOEXEC, 0);
+  // TODO: use wlr_allocator_autocreate or retrieve a reference from wlr_renderer once possible
+  if (wlr_renderer_is_gles2 (self->wlr_renderer))
+    self->allocator = (struct wlr_allocator*) wlr_gbm_allocator_create (drm_fd);
+}
+
+
+static void
+phoc_renderer_finalize (GObject *object)
+{
+  PhocRenderer *self = PHOC_RENDERER (object);
+  if (self->allocator)
+    wlr_allocator_destroy (self->allocator);
+  /* TODO: destroy wlr_renderer */
+}
+
+
+static void
+phoc_renderer_class_init (PhocRendererClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->get_property = phoc_renderer_get_property;
+  object_class->set_property = phoc_renderer_set_property;
+  object_class->constructed = phoc_renderer_constructed;
+  object_class->finalize = phoc_renderer_finalize;
+
+  /**
+   * PhocRenderer:wlr-renderer
+   *
+   * The wlr-renderer from wlroots to use
+   */
+  props[PROP_WLR_RENDERER] =
+    g_param_spec_pointer ("wlr-renderer",
+                          "",
+                          "",
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
+
+  /**
+   * PhocRenderer::render-start
+   * @self: The renderer emitting the signal
+   * @output: The output being rendered on
+   *
+   * This signal is emitted at the start of a render pass
+   */
+  signals[RENDER_START] = g_signal_new ("render-start",
+                                        G_TYPE_FROM_CLASS (klass),
+                                        G_SIGNAL_RUN_LAST,
+                                        0, NULL, NULL, NULL,
+                                        G_TYPE_NONE, 1, PHOC_TYPE_OUTPUT);
+  /**
+   * PhocRenderer::render-end
+   * @self: The renderer emitting the signal
+   * @output: The output being rendered on
+   *
+   * This signal is emitted at the end of a render pass
+   */
+  signals[RENDER_END] = g_signal_new ("render-end",
+                                      G_TYPE_FROM_CLASS (klass),
+                                      G_SIGNAL_RUN_LAST,
+                                      0, NULL, NULL, NULL,
+                                      G_TYPE_NONE, 1, PHOC_TYPE_OUTPUT);
+}
+
+
+static void
+phoc_renderer_init (PhocRenderer *self)
+{
+}
+
+
+PhocRenderer *
+phoc_renderer_new (struct wlr_renderer *wlr_renderer)
+{
+  return PHOC_RENDERER (g_object_new (PHOC_TYPE_RENDERER,
+                                      "wlr-renderer", wlr_renderer,
+                                      NULL));
 }
