@@ -1,10 +1,11 @@
-#ifndef ROOTSTON_SWITCH_H
-#define ROOTSTON_SWITCH_H
+#pragma once
 
 #include "input.h"
 
+G_BEGIN_DECLS
+
 struct roots_switch {
-	struct roots_seat *seat;
+	PhocSeat *seat;
 	struct wlr_input_device *device;
 	struct wl_listener device_destroy;
 
@@ -15,4 +16,4 @@ struct roots_switch {
 void roots_switch_handle_toggle(struct roots_switch *switch_device,
 		struct wlr_event_switch_toggle *event);
 
-#endif
+G_END_DECLS
