@@ -24,7 +24,7 @@ abgr_to_argb (PhocTestBuffer *buffer)
 		 buffer->format == WL_SHM_FORMAT_XBGR8888);
   guint8 *data = buffer->shm_data;
 
-  for (int i = 0; i < buffer->height * buffer->stride / 4; i += 4) {
+  for (int i = 0; i < buffer->height * buffer->stride; i += 4) {
       guint32 *px = (guint32 *)(data + i);
       guint8 r, g, b, a;
 
@@ -308,7 +308,7 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
     xdg_wm_base_add_listener(globals->xdg_shell, &wm_base_listener, NULL);
   } else if (!g_strcmp0 (interface, zwlr_layer_shell_v1_interface.name)) {
     globals->layer_shell = wl_registry_bind (registry, name,
-					     &zwlr_layer_shell_v1_interface, 1);
+					     &zwlr_layer_shell_v1_interface, 2);
   } else if (!g_strcmp0 (interface, zwlr_screencopy_manager_v1_interface.name)) {
     globals->screencopy_manager = wl_registry_bind (registry, name,
 						    &zwlr_screencopy_manager_v1_interface, 1);
@@ -318,7 +318,9 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
     zwlr_foreign_toplevel_manager_v1_add_listener (globals->foreign_toplevel_manager,
 						   &foreign_toplevel_manager_listener, globals);
   } else if (!g_strcmp0 (interface, phosh_private_interface.name)) {
-    globals->phosh = wl_registry_bind (registry, name, &phosh_private_interface, 5);
+    globals->phosh = wl_registry_bind (registry, name, &phosh_private_interface, 6);
+  } else if (!g_strcmp0 (interface, gtk_shell1_interface.name)) {
+    globals->gtk_shell1 = wl_registry_bind (registry, name, &gtk_shell1_interface, 3);
   }
 }
 
@@ -351,6 +353,7 @@ wl_client_run (GTask *task, gpointer source,
   g_assert_nonnull (globals.layer_shell);
   g_assert_nonnull (globals.shm);
   g_assert_nonnull (globals.xdg_shell);
+  g_assert_nonnull (globals.gtk_shell1);
 
   g_assert (globals.formats & (1 << WL_SHM_FORMAT_XRGB8888));
 
@@ -410,6 +413,7 @@ phoc_test_client_run (gint timeout, PhocTestClientIface *iface, gpointer data)
 
   g_assert_true (PHOC_IS_SERVER (server));
   g_assert_true (phoc_server_setup(server, TEST_PHOC_INI, NULL, loop,
+                                   PHOC_SERVER_FLAG_NONE,
 				   PHOC_SERVER_DEBUG_FLAG_NONE));
   if (iface && iface->server_prepare)
     g_assert_true (iface->server_prepare(server, data));
@@ -658,10 +662,11 @@ gboolean
 phoc_test_buffer_matches_screenshot (PhocTestBuffer *buffer, const gchar *filename)
 {
   const char *msg;
-  cairo_surface_t *surface = surface = cairo_image_surface_create_from_png (filename);
+  cairo_surface_t *surface = cairo_image_surface_create_from_png (filename);
   cairo_format_t format;
   guint32 *l, *r;
   guint32 mask = 0xFFFFFFFF;
+  int ret;
 
   g_assert_true (buffer->format == WL_SHM_FORMAT_XRGB8888
 		 || buffer->format == WL_SHM_FORMAT_ARGB8888);
@@ -701,7 +706,7 @@ phoc_test_buffer_matches_screenshot (PhocTestBuffer *buffer, const gchar *filena
   if (buffer->height != cairo_image_surface_get_height (surface) ||
       buffer->width != cairo_image_surface_get_width (surface) ||
       buffer->stride != cairo_image_surface_get_stride (surface)) {
-    g_debug ("Metadata mismatch");
+    g_test_message ("Metadata mismatch for %s", filename);
     return FALSE;
   }
 
@@ -709,13 +714,14 @@ phoc_test_buffer_matches_screenshot (PhocTestBuffer *buffer, const gchar *filena
   r = (guint32*)cairo_image_surface_get_data (surface);
   g_assert_nonnull (r);
 
+  ret = TRUE;
   for (int i = 0; i < buffer->height * buffer->stride / 4; i++) {
     if ((l[i] & mask) != (r[i] & mask)) {
-      g_debug ("Mismatch: %d: 0x%x 0x%x", i, l[i], r[i]);
-      return FALSE;
+      g_test_message ("Mismatch: %d: 0x%x 0x%x for %s", i, l[i], r[i], filename);
+      ret = FALSE;
     }
   }
-  return TRUE;
+  return ret;
 }
 
 void
